/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "build.h"

string get_configure_parameters();

// Run the autogen.sh command if a project is currently open.
void openldev_autogen (GtkMenuItem *menuitem,
                       gpointer data)
{  
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->project->get_name().empty())
    error("Please open or create\na project first!");
  else 
  {
    gtk_notebook_set_current_page(GTK_NOTEBOOK(ptr->output->get_notebook()), 0);
    chdir (ptr->project->get_location().c_str());
    
    string command = string(ptr->ptemplate->get_autogen()) + " 2>&1";
    ptr->output->get_build()->run_command(command, ptr->env);
  }
}

// Compile current file if a file and a project is open. The function then
// creates the compile string from the project language & runs the command
void openldev_compile (GtkMenuItem *menuitem,
                       gpointer data)
{  
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (!ptr->noteman->noteman_get_file_open())
    error("Please open or create\na file first!");
  else 
  {
    gtk_notebook_set_current_page(GTK_NOTEBOOK(ptr->output->get_notebook()), 0);
    string command = ptr->ptemplate->get_compile(ptr->noteman->noteman_get_current_filename());
    string location = string(ptr->project->get_location()) + "src/";
    chdir (ptr->project->get_location().c_str());

    command += " 2>&1";
    ptr->output->get_build()->run_command(command, ptr->env);
  }
}

// If a project is open, build it. Run autogen.sh and then build again if
// Makefiles are not found. All of the files are first saved.
void openldev_build (GtkMenuItem *menuitem,
                     gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->project->get_name().empty())
    error("Please open or create\na project first!");
  else 
  {
    gtk_notebook_set_current_page(GTK_NOTEBOOK(ptr->output->get_notebook()), 0);
    ptr->noteman->noteman_save_all_files();
    chdir (ptr->project->get_location().c_str());
    
    string command = string(ptr->ptemplate->get_build()) + " 2>&1";
    if (!ptr->output->get_build()->run_command(command, ptr->env))
    {
      openldev_autogen (menuitem, data);
      ptr->output->get_build()->run_command(command, ptr->env);
    }
  }
}

// If a project is open, run the executable in the terminal specified
// by the user 
void openldev_execute (GtkMenuItem *menuitem,
                       gpointer data)
{  
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->project->get_name().empty())
    error("Please open or create\na project first!");
  else 
  {
    string command = string(ptr->project->get_location())
      + "src/" + ptr->project->get_name() + "\n";
    gtk_notebook_set_current_page(GTK_NOTEBOOK(ptr->output->get_notebook()), 2);
    ptr->output->run_terminal_command(command);
  }
}

// If a project is open, run the ./configure command in the project directory
void openldev_configure (GtkMenuItem *menuitem,
                         gpointer data)
{  
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->project->get_name().empty())
    error("Please open or create\na project first!");
  else 
  {
    string parameters = get_configure_parameters();
    if (parameters != "CANCEL")
    {
      gtk_notebook_set_current_page(GTK_NOTEBOOK(ptr->output->get_notebook()), 0);
      chdir (ptr->project->get_location().c_str());
    
      string command = string(ptr->ptemplate->get_configure()) + " " + parameters + " 2>&1";
      ptr->output->get_build()->run_command(command, ptr->env);
    }
  }
}

// Clean all of the sources, build files, object files and those files
// created by configure
void openldev_clean (GtkMenuItem *menuitem,
                     gpointer data)
{  
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->project->get_name().empty())
    error("Please open or create\na project first!");
  else 
  {
    gtk_notebook_set_current_page(GTK_NOTEBOOK(ptr->output->get_notebook()), 0);
    chdir (ptr->project->get_location().c_str());
    
    string command = string(ptr->ptemplate->get_clean()) + " 2>&1";
    ptr->output->get_build()->run_command(command, ptr->env);
  }
}

// Stop the current compilation, clean, configure, autogen or build
void openldev_stop (GtkMenuItem *menuitem,
                       gpointer data)
{  
  OpenLDev *ptr = (OpenLDev*) data;
  ptr->output->get_build()->stop_build();
}

// Create a dialog that will ask the user for parameters to send to the
// ./configure command. It returns the parameters even if they are empty.
// The string "CANCEL" is returned if the user chooses to Cancel.
string get_configure_parameters()
{
  string return_text;
  GtkWidget *dialog, *cancel, *ok, *label, *parameters, *hbox;
  dialog = gtk_dialog_new ();
  cancel = gtk_button_new_from_stock ("gtk-cancel");
  ok = gtk_button_new_from_stock ("gtk-ok");
  label = gtk_label_new ("Parameters:");
  parameters = gtk_entry_new ();
  hbox = gtk_hbox_new (false, 10);
  
  GTK_WIDGET_SET_FLAGS (ok, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX(hbox), label, false, false, 0);
  gtk_box_pack_start (GTK_BOX(hbox), parameters, true, true, 0);
  
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox),
                      hbox, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER(dialog), 5);
  gtk_widget_set_size_request (dialog, 350, 100);
  gtk_window_set_title (GTK_WINDOW(dialog), "Configure Parameters");
  gtk_window_set_modal (GTK_WINDOW(dialog), true);
  gtk_window_set_resizable (GTK_WINDOW(dialog), false);
  gtk_dialog_set_has_separator (GTK_DIALOG(dialog), true);
  gtk_dialog_add_action_widget (GTK_DIALOG(dialog), cancel, GTK_RESPONSE_CANCEL);
  gtk_dialog_add_action_widget (GTK_DIALOG(dialog), ok, GTK_RESPONSE_OK);
  gtk_widget_show_all (dialog);

  switch (gtk_dialog_run(GTK_DIALOG(dialog)))
  {
    case (GTK_RESPONSE_OK):
      return_text = gtk_entry_get_text (GTK_ENTRY(parameters));
      break;
    default:
      return_text = "CANCEL";
      break;
  }
  
  gtk_widget_hide (dialog);
  return return_text;
}
