/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "gotoline.h"

// This function creates the goto line toolbar by initializing the widgets, 
// creating the buttons and adding them to the horizontal box. The buttons are
// then connected to their signals
void OpenLDev::openldev_create_goto_line_toolbar()
{
  gototoolbar = new GotoLineToolbar();
  
  // Connect the buttons to their signals
  g_signal_connect(G_OBJECT(gototoolbar->get_close()), "clicked",
                   G_CALLBACK(on_close_toolbars), (gpointer)this);
  g_signal_connect(G_OBJECT(gototoolbar->get_goto()), "clicked",
                   G_CALLBACK(on_jump_to_line_number), (gpointer)this);
}

// Toggle the visibility of the goto line toolbar. Also, this makes sure that if
// the goto line toolbar is visible, the find toolbar is hidden
void openldev_goto_line (GtkMenuItem *menuitem,
                         gpointer data) 
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (GTK_WIDGET_VISIBLE(ptr->gototoolbar->get_hbox()))
    on_jump_to_line_number(ptr->gototoolbar->get_goto(), data);
  else
  {
    gtk_widget_show(ptr->gototoolbar->get_hbox());
    gtk_widget_hide(ptr->findtoolbar->get_hbox());
  }
}

// Close both toolbars. The find toolbar and the goto line toolbar.
void on_close_toolbars (GtkWidget *widget, 
                        gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  gtk_widget_hide(ptr->gototoolbar->get_hbox());
  gtk_widget_hide(ptr->findtoolbar->get_hbox());
}

// Move to the specified line number in the buffer. If the line exists, the
// cursor is moved to the beginning of that line and the notebookmanager scrolls
// to it. If the line does not exist (e.g. It is out of range), then the goto
// line error message and image are shown, not to be hidden until a successful
// goto line is completed.
void on_jump_to_line_number (GtkWidget *widget,
                             gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->noteman->noteman_get_file_open())
  {
    GtkTextBuffer *buffer = GTK_TEXT_BUFFER(ptr->noteman->noteman_get_textbuffer());
  
    // If the line is within the bounds of the document
    if (gtk_text_buffer_get_line_count(buffer) >= ptr->gototoolbar->get_goto_value())
    {
      GtkTextIter iter;
      gtk_text_buffer_get_iter_at_line(buffer, &iter, ptr->gototoolbar->get_goto_value()-1);
      gtk_text_buffer_move_mark_by_name(buffer, "insert", &iter);
      gtk_text_buffer_move_mark_by_name(buffer, "selection_bound", &iter);
      ptr->noteman->noteman_scroll_to_mark();
      gtk_widget_grab_focus(ptr->noteman->noteman_get_textview());
    
      ptr->gototoolbar->show_error(false);
    }
    // If the line is out of bounds...
    else
      ptr->gototoolbar->show_error(true);
  }
}
